#include "aqualooks_draw.h"
#include "aqualooks_style.h"
#include "aqualooks_types.h"

#include "support.h"

#include <cairo.h>

#define M_PI 3.14159265358979323846

void
aqualooks_rounded_rectangle (cairo_t *cr,
                                 double x, double y, double w, double h,
                                 double radius, uint8 corners)
{
	if (corners & CL_CORNER_TOPLEFT)
		cairo_move_to (cr, x+radius, y);
	else
		cairo_move_to (cr, x, y);
	
	if (corners & CL_CORNER_TOPRIGHT)
		cairo_arc (cr, x+w-radius, y+radius, radius, M_PI * 1.5, M_PI * 2);
	else
		cairo_line_to (cr, x+w, y);
	
	if (corners & CL_CORNER_BOTTOMRIGHT)
		cairo_arc (cr, x+w-radius, y+h-radius, radius, 0, M_PI * 0.5);
	else
		cairo_line_to (cr, x+w, y+h);
	
	if (corners & CL_CORNER_BOTTOMLEFT)
		cairo_arc (cr, x+radius,   y+h-radius, radius, M_PI * 0.5, M_PI);
	else
		cairo_line_to (cr, x, y+h);
	
	if (corners & CL_CORNER_TOPLEFT)
		cairo_arc (cr, x+radius,   y+radius,   radius, M_PI, M_PI * 1.5);
	else
		cairo_line_to (cr, x, y);
}

//Used for push buttons only
static void
aqualooks_rounded_rectangle2 (cairo_t *cr, double x, double y, double width, double height)
{
	cairo_arc (cr, x+height/2, y, height/2, 90*M_PI/180, 270*M_PI/180);
	cairo_arc (cr, x+width - height/2, y, height/2, 270*M_PI/180, 90*M_PI/180);
	cairo_line_to  (cr, x+height/2, y+height/2);
	cairo_close_path(cr);

}


/* KEEP IN MIND, MIRROR TAKES PLACE BEFORE ROTATION */
/* ROTATES ANTI-CLOCKWISE, I THINK :P */
/* TODO: Do I really need THREE matrices? */
static void
rotate_mirror_translate (cairo_t *cr, double radius, double x, double y,
                         boolean mirror_horizontally, boolean mirror_vertically)
{
	cairo_matrix_t matrix_rotate;
	cairo_matrix_t matrix_mirror;
	cairo_matrix_t matrix_result;
	
	double r_cos = cos(radius);
	double r_sin = sin(radius);
	
	cairo_matrix_init (&matrix_rotate, r_cos,
	                                   r_sin,
	                                   r_sin,
	                                   r_cos,
	                                   x, y);
	
	cairo_matrix_init (&matrix_mirror, mirror_horizontally ? -1 : 1,
	                                   0,
	                                   0,
	                                   mirror_vertically ? -1 : 1,
									   0, 0);

	cairo_matrix_multiply (&matrix_result, &matrix_mirror, &matrix_rotate);

	cairo_set_matrix (cr, &matrix_result);
}

void
aqualooks_draw_inset (cairo_t *cr, int width, int height,
                       double radius, uint8 corners)
{
	double top_x1 = 0, top_x2 = width, bot_x1 = 0, bot_x2 = width;
	
	if (corners & CL_CORNER_TOPLEFT)
		top_x1 = radius-1;
	
	if (corners & CL_CORNER_TOPRIGHT)
		top_x2 = width-radius+1;
	
	if (corners & CL_CORNER_BOTTOMLEFT)
		bot_x1 = radius-1;
	
	if (corners & CL_CORNER_BOTTOMRIGHT)
		bot_x2 = width-radius+1;
	
	cairo_set_line_width (cr, 1);
	cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.04);
	cairo_move_to (cr, top_x1, 0.0);
	cairo_line_to (cr, top_x2, 0.0);
	cairo_stroke (cr);
	
	cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.5);
	cairo_move_to (cr, bot_x1, height);
	cairo_line_to (cr, bot_x2, height);
	cairo_stroke (cr);
}

static void
aqualooks_draw_shadow (cairo_t *cr, int width, int height)
{
	cairo_set_line_width (cr, 1.0);
	
	cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.1);
	
	cairo_move_to (cr, width -2, 3.0);
	//cairo_arc (cr, width-4.0, height-4.0, 4.0, 0, M_PI/2);
	//cairo_line_to (cr, 3.0, height);

	cairo_stroke (cr);
}

void aqualooks_draw_gradients(cairo_t *cr,int xoffset,int yoffset,int width,int height)
{
        //Gradients:
        cairo_pattern_t *pat;
        pat = cairo_pattern_create_linear (xoffset, yoffset,  xoffset, height);
        cairo_pattern_add_color_stop_rgba (pat, 0, 1, 1, 1, 0);
        cairo_pattern_add_color_stop_rgba (pat, 0.5, 0.3, 0.3, 0.3, 0.2);
	cairo_pattern_add_color_stop_rgba (pat, 1, 1, 1, 1, 0.3);
        cairo_rectangle(cr, xoffset + 1.25, yoffset + 1.25, width-(xoffset*2)-2.5, height-(yoffset*2)-2.5);
	cairo_set_source (cr, pat);
	cairo_fill (cr);
	cairo_pattern_destroy (pat);
	

        pat = cairo_pattern_create_linear (xoffset, yoffset,  xoffset, height);
        cairo_pattern_add_color_stop_rgba (pat, 0, 1, 1, 1, 0.5);
	cairo_pattern_add_color_stop_rgba (pat, 1, 1, 1, 1, 0);
        cairo_rectangle(cr, xoffset + 1.25, yoffset + 1.25, width-(xoffset*2)-2.5, height-(yoffset*2)-2.5);
	cairo_set_source (cr, pat);
	cairo_fill (cr);
	cairo_pattern_destroy (pat);

	//end of grads
}

void aqualooks_draw_gradients_inwards(cairo_t *cr,int xoffset,int yoffset,int width,int height, 
					double radius, uint8 corners)
{
        //Gradients:
        cairo_pattern_t *pat;
        pat = cairo_pattern_create_linear (xoffset, yoffset,  xoffset, height);
        cairo_pattern_add_color_stop_rgba (pat, 0, 1, 1, 1, 0);
        cairo_pattern_add_color_stop_rgba (pat, 0.5, 0.3, 0.3, 0.3, 0.2);
	cairo_pattern_add_color_stop_rgba (pat, 1, 1, 1, 1, 0.3);
        //cairo_rectangle(cr, xoffset + 1.25, yoffset + 1.25, width-(xoffset*2)-2.5, height-(yoffset*2)-2.5);
	aqualooks_rounded_rectangle(cr, xoffset + 1.25, yoffset + 1.25, width-(xoffset*2)-2.5, height-(yoffset*2)-2.5, radius, corners);
	cairo_set_source (cr, pat);
	cairo_fill (cr);
	cairo_pattern_destroy (pat);
	

        pat = cairo_pattern_create_linear (xoffset, yoffset,  xoffset, height);
        cairo_pattern_add_color_stop_rgba (pat, 0, 1, 1, 1, 0.5);
	cairo_pattern_add_color_stop_rgba (pat, 1, 1, 1, 1, 0);
        //cairo_rectangle(cr, xoffset + 1.25, yoffset + 1.25, width-(xoffset*2)-2.5, height-(yoffset*2)-2.5);
	aqualooks_rounded_rectangle(cr, xoffset + 1.25, yoffset + 1.25, width-(xoffset*2)-2.5, height-(yoffset*2)-2.5, radius, corners);
	cairo_set_source (cr, pat);
	cairo_fill (cr);
	cairo_pattern_destroy (pat);

	//end of grads
}

static void
aqualooks_draw_top_left_highlight (cairo_t *cr,
                                    const WidgetParameters *params,
                                    int width, int height, gdouble radius)
{
	double light_y1 = params->ythickness-1,
	       light_y2 = height - params->ythickness - 1,
	       light_x1 = params->xthickness-1,
	       light_x2 = width - params->xthickness - 1;

	cairo_move_to         (cr, light_x1, light_y2);
	
	if (params->corners & CL_CORNER_TOPLEFT)
		cairo_arc         (cr, light_x1+radius, light_y1+radius, radius, M_PI, 270*(M_PI/180));
	else
		cairo_line_to     (cr, light_x1, light_y1);
	
	cairo_line_to         (cr, light_x2, light_y1);
	cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.6);
	cairo_stroke          (cr);
}

#warning seems to be very slow in scrollbar_stepper
static void
aqualooks_draw_highlight_and_ul_shade (cairo_t *cr,
                                     const ShadowParameters *params,
                                     int width, int height, gdouble radius)
{
	uint8 corners = params->corners;
	double x = 1.0;
	double y = 1.0;
	
	width  -= 3;
	height -= 3;
	
	cairo_save (cr);
	
	/* Draw top/left */
	if (corners & CL_CORNER_BOTTOMLEFT)
		cairo_move_to (cr, x, y+height-radius);
	else
		cairo_move_to (cr, x, y+height);
	
	if (corners & CL_CORNER_TOPLEFT)
		cairo_arc (cr, x+radius, y+radius, radius, M_PI, 270*(M_PI/180));
	else
		cairo_line_to (cr, x, y);
	
	if (corners & CL_CORNER_TOPRIGHT)
		cairo_line_to (cr, x+width-radius, y);
	else
		cairo_line_to (cr, x+width, y);
	
	if (params->shadow & CL_SHADOW_OUT)
		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.5);
	else
		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.05);
		
	cairo_stroke (cr);
	
	/* Drop bottom/right */
	
	if (corners & CL_CORNER_TOPRIGHT)
	{
		cairo_move_to (cr, x+width-radius, y);
		cairo_arc (cr, x+width-radius, y+radius, radius, M_PI + M_PI*0.5, M_PI*2);
	}
	else
		cairo_move_to (cr, x+width, y);
	
	if (corners & CL_CORNER_BOTTOMRIGHT)
		cairo_arc (cr, x+width-radius, y+height-radius, radius, 0, M_PI/2);
	else
		cairo_line_to (cr, x+width, y+height);
	
	if (corners & CL_CORNER_BOTTOMLEFT)
		cairo_arc (cr, x+radius, y+height-radius, radius, M_PI/2, M_PI);
	else
		cairo_line_to (cr, x, y+height);
	
	if (params->shadow & CL_SHADOW_OUT)
		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.05);
	else
		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.5);
	
	cairo_stroke (cr);
	
	cairo_restore (cr);
}

void
aqualooks_draw_gripdots (cairo_t *cr, int x, int y,
                          int width, int height, int xr, int yr,
                          float contrast)
{
	int i, j;
	int xoff, yoff;

	for ( i = 0; i < xr; i++ ) {
		for ( j = 0; j < yr; j++ ) {
			xoff = x -(xr * 3 / 2) + 3 * i;
			yoff = y -(yr * 3 / 2) + 3 * j; 
			
			cairo_rectangle (cr, width/2+0.5+xoff, height/2+0.5+yoff, 2, 2);
			cairo_set_source_rgba (cr, 1., 1., 1., 0.8+contrast);
			cairo_fill (cr);
			cairo_rectangle (cr, width/2+0.5+xoff, height/2+0.5+yoff, 1, 1);
			cairo_set_source_rgba (cr, 0, 0, 0, 0.2+contrast);
			cairo_fill (cr);
		}
	}
}

static void
aqualooks_draw_button_gradient (cairo_t *cr,
                                  double x, double y, int width, int height,
                                  CairoColor *color, boolean fill, boolean disabled, double radius, uint8 corners)
{
	CairoColor a, b, c, d, e,f,g,h,i,j,k;
	cairo_pattern_t *pt;

	if (fill)
	{
		ul_shade (color, &a, 1.12);
		ul_shade (color, &b, 1.00);
		ul_shade (color, &c, 0.96);
		ul_shade (color, &d, 0.92);
		ul_shade (color, &e, 0.74);
		ul_shade (color, &f, 0.88 );
		ul_shade (color, &g, 0.96 );
		ul_shade (color, &h, 1.24 );


	} else {
		ul_shade (color, &a, disabled? 1.03 : 1.15);
		ul_shade (color, &b, disabled? 1.01 : 1.02);
		ul_shade (color, &c, disabled? 0.99 : 0.96);
		ul_shade (color, &d, 0.92 );
		ul_shade (color, &e, disabled? 1.01 : 0.74);
		ul_shade (color, &f, disabled? 1.01 : 0.88);
		ul_shade (color, &g, disabled? 1.01 : 0.96);
		ul_shade (color, &h, disabled? 1.01 : 1.24);

	}

	pt = cairo_pattern_create_linear (x, y, x, y+height);
	cairo_pattern_add_color_stop_rgb (pt, 0.0,  a.r, a.g, a.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.2,  b.r, b.g, b.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.3,  c.r, c.g, c.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.45, d.r, d.g, d.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.45, e.r, e.g, e.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.65, f.r, f.g, f.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.85, g.r, g.g, g.b);
	cairo_pattern_add_color_stop_rgb (pt, 1.0,  h.r, h.g, h.b);


	cairo_set_source (cr, pt);
	aqualooks_rounded_rectangle (cr, x, y, width, height, radius, corners);

	cairo_fill (cr);
	
	cairo_pattern_destroy (pt);
}

void
aqualooks_draw_button (cairo_t *cr,
                        const AqualooksColors *colors,
                        const WidgetParameters *params,
                        const ShadowParameters *shadow,
                        int x, int y, int width, int height, 
			double radius)
{
	const float RADIUS = radius;

	double xoffset = 0, yoffset = 0;
	CairoColor fill                   = colors->bg[params->state_type];
	const CairoColor *border_normal   = &colors->shade[7];
	const CairoColor *border_disabled = &colors->shade[4];
	const CairoColor *gradient_bottom = &colors->shade[3];

	cairo_pattern_t *pattern;
	
	cairo_translate (cr, x, y);
	cairo_set_line_width (cr, 1.0);

	if (params->xthickness == 3 || params->ythickness == 3)
	{
		cairo_translate (cr, 0.5, 0.5);
		
		if (params->prelight && params->enable_glow)
		{
			const CairoColor *glow = &colors->spot[0];
			aqualooks_rounded_rectangle (cr, 0, 0, width-1, height-1, RADIUS+1, params->corners);
			cairo_set_source_rgb (cr, glow->r, glow->g, glow->b);
			cairo_stroke (cr);
		}
		if (!params->active && shadow->shadow != CL_SHADOW_IN)
			aqualooks_draw_shadow (cr, width-1, height-1);
		//else
		//	aqualooks_draw_inset (cr, width-1, height-1, RADIUS, params->corners);
		
		cairo_translate (cr, -0.5, -0.5);
		
		if (params->xthickness == 3)
			xoffset = 1;
		if (params->ythickness == 3)
			yoffset = 1;
	}
	
	if (params->active && params->prelight)
	{
		fill = colors->bg[CL_STATE_ACTIVE];
		ul_shade (&fill, &fill, 1.05);
	}

	aqualooks_draw_button_gradient (cr, xoffset+1, yoffset+1, width-(xoffset*2)-2, height-(yoffset*2)-2, &fill, FALSE, params->disabled, RADIUS+1, params->corners);
	
	if (params->active)
	{

		pattern = cairo_pattern_create_linear (xoffset+1, yoffset+1, xoffset+1, yoffset+4);
		cairo_pattern_add_color_stop_rgba (pattern, 0.0, 0.0, 0.0, 0.0, params->disabled ? 0.08 : 0.20);
		cairo_pattern_add_color_stop_rgba (pattern, 1.0, 0.0, 0.0, 0.0, 0.50);
		cairo_set_source (cr, pattern);
		cairo_fill (cr);
		cairo_pattern_destroy (pattern);
	}
	
	// Draw the default inside "glow" 
	if (params->is_default && !params->active && !params->disabled)
	{
		const CairoColor *glow = &colors->spot[0];
		double hh = 0;

		cairo_set_line_width (cr, 1);		
	  	aqualooks_rounded_rectangle (cr, 3.5, 3.5, width-7, height-7, RADIUS, params->corners);
		cairo_set_source_rgb (cr, glow->r, glow->g, glow->b);
		cairo_stroke (cr);

		cairo_set_line_width (cr, 1.5);
		glow = &colors->spot[0];
		cairo_set_line_width (cr, 1.5);		
	  	aqualooks_rounded_rectangle (cr, 2.5, 2.5+hh, width-5.5, height-5.5, RADIUS, params->corners);
		cairo_set_source_rgb (cr, glow->r, glow->g, glow->b);
		cairo_stroke (cr);
		
		hh--;

		cairo_set_line_width (cr, 2);
		glow = &colors->spot[1];
		cairo_set_line_width (cr, 1.5);		
	  	aqualooks_rounded_rectangle (cr, 2.5, 2.5+hh, width-4, height-4, RADIUS, params->corners);
		cairo_set_source_rgb (cr, glow->r, glow->g, glow->b);
		cairo_stroke (cr);

	}

	cairo_set_line_width (cr, 1.5);
	// Draw the border
	if (params->is_default || (params->prelight && params->enable_glow))
		border_normal = &colors->spot[2];
	
	if (params->disabled)
	{
		cairo_set_source_rgb (cr, border_disabled->r,
		                          border_disabled->g,
		                          border_disabled->b);
	}
	else
	{
		cairo_set_source_rgb (cr, border_normal->r,
		                          border_normal->g,
		                          border_normal->b);
	}
	
	aqualooks_rounded_rectangle (cr, xoffset + 0.5, yoffset + 0.5,
                                  width-(xoffset*2)-1, height-(yoffset*2)-1,
                                  RADIUS, params->corners);
	cairo_stroke (cr);
	

}

static void
aqualooks_draw_pushbutton_gradient (cairo_t *cr,
                                  double x, double y, int width, int height,
                                  CairoColor *color, boolean fill, boolean disabled, double radius, uint8 corners)
{
	CairoColor a, b, c, d, e,f,g,h,i,j,k;
	cairo_pattern_t *pt;

	if (fill)
	{
		ul_shade (color, &a, 1.12);
		ul_shade (color, &b, 1.00);
		ul_shade (color, &c, 0.96);
		ul_shade (color, &d, 0.92);
		ul_shade (color, &e, 0.74);
		ul_shade (color, &f, 0.88 );
		ul_shade (color, &g, 0.96 );
		ul_shade (color, &h, 1.24 );


	} else {
		ul_shade (color, &a, disabled? 1.03 : 1.15);
		ul_shade (color, &b, disabled? 1.01 : 1.02);
		ul_shade (color, &c, disabled? 0.99 : 0.96);
		ul_shade (color, &d, 0.92 );
		ul_shade (color, &e, disabled? 1.01 : 0.74);
		ul_shade (color, &f, disabled? 1.01 : 0.88);
		ul_shade (color, &g, disabled? 1.01 : 0.96);
		ul_shade (color, &h, disabled? 1.01 : 1.24);

	}

	pt = cairo_pattern_create_linear (x, y, x, y+height);
	cairo_pattern_add_color_stop_rgb (pt, 0.0,  a.r, a.g, a.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.2,  b.r, b.g, b.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.3,  c.r, c.g, c.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.45, d.r, d.g, d.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.45, e.r, e.g, e.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.65, f.r, f.g, f.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.85, g.r, g.g, g.b);
	cairo_pattern_add_color_stop_rgb (pt, 1.0,  h.r, h.g, h.b);


	cairo_set_source (cr, pt);
	//aqualooks_rounded_rectangle (cr, x, y, width, height, radius, corners);
	aqualooks_rounded_rectangle2 (cr, x, y + height/2, width, height);

	cairo_fill (cr);
	
	cairo_pattern_destroy (pt);
}

void
aqualooks_draw_push_button (cairo_t *cr,
                        const AqualooksColors *colors,
                        const WidgetParameters *params,
                        const ShadowParameters *shadow,
                        int x, int y, int width, int height, 
			double radius)
{

	const float RADIUS = radius;

	double xoffset = 0, yoffset = 0;
	CairoColor fill                   = colors->bg[params->state_type];
	const CairoColor *border_normal   = &colors->shade[7];
	const CairoColor *border_disabled = &colors->shade[4];
	const CairoColor *gradient_bottom = &colors->shade[3];

	CairoColor border, top, bottom;
	border = top = bottom = colors->spot[1];

	cairo_pattern_t *pattern;
	
	cairo_translate (cr, x, y);
	cairo_set_line_width (cr, 1.0);

	if (params->xthickness == 3 || params->ythickness == 3)
	{
		cairo_translate (cr, 0.5, 0.5);
		
		if (params->prelight && params->enable_glow)
		{
			const CairoColor *glow = &colors->spot[0];
			aqualooks_rounded_rectangle2 (cr, x, y + (height-1)/2, width-3, height-1);
			cairo_set_source_rgb (cr, glow->r, glow->g, glow->b);
			cairo_stroke (cr);
		}

		
		cairo_translate (cr, -0.5, -0.5);
		
		if (params->xthickness == 3)
			xoffset = 1;
		if (params->ythickness == 3)
			yoffset = 1;
	}
	
	if ((params->active && params->prelight))
	{
		fill = colors->bg[CL_STATE_PRELIGHT];
		ul_shade (&fill, &fill, 1.05);
	}

	if ((params->is_default && !params->disabled))
	{
		fill = colors->bg[CL_STATE_ACTIVE];
		ul_shade (&fill, &fill, 1.05);
	}

	// Draw shadow
	cairo_set_line_width (cr, 2.0);

	if (params->prelight)
	{
		border = colors->spot[2];
	}
	else
	{
		border = colors->base[params->state_type];
		ul_shade(&border, &border, 0.5);
	}



	aqualooks_rounded_rectangle2 (cr, xoffset-0.40, yoffset+ (height - 3.5)/2 +2.0, width - 2, height - 3.5);

	cairo_set_source_rgb (cr, border.r, border.g, border.b);
	cairo_stroke (cr);
		
	aqualooks_rounded_rectangle2 (cr, xoffset-0.50, yoffset+ (height - 4.5)/2 +3.0, width - 3.5, height - 4.5);
	cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.4);
	cairo_stroke (cr);


	aqualooks_draw_pushbutton_gradient (cr, xoffset+1, yoffset+1, width-(xoffset*2)-3, height-(yoffset*2)-2, &fill, FALSE, params->disabled, RADIUS, params->corners);

	
	if (params->active || params->is_default)
	{

		pattern = cairo_pattern_create_linear (xoffset+1, yoffset+1, xoffset+1, yoffset+4);
		cairo_pattern_add_color_stop_rgba (pattern, 0.0, 0.0, 0.0, 0.0, params->disabled ? 0.08 : 0.20);
		cairo_pattern_add_color_stop_rgba (pattern, 1.0, 0.0, 0.0, 0.0, 0.50);
		cairo_set_source (cr, pattern);
		cairo_fill (cr);
		cairo_pattern_destroy (pattern);
	}
	

	cairo_set_line_width (cr, 1.25);
	// Draw the border
	if (params->is_default || (params->prelight && params->enable_glow))
		border_normal = &colors->spot[2];
	
	if (params->disabled)
	{
		cairo_set_source_rgb (cr, border_disabled->r,
		                          border_disabled->g,
		                          border_disabled->b);
	}
	else
	{
		cairo_set_source_rgb (cr, border_normal->r,
		                          border_normal->g,
		                          border_normal->b);
	}
	
	aqualooks_rounded_rectangle2 (cr, xoffset + 1.0, yoffset + (height-(yoffset*2)-1)/2 +0.5,
                                  width-(xoffset*2)-2.25, height-(yoffset*2)-1);
	cairo_stroke (cr);

}






void
aqualooks_draw_entry (cairo_t *cr,
                       const AqualooksColors *colors,
                       const WidgetParameters *params,
                       int x, int y, int width, int height)
{
	const float RADIUS = 4.0;
	
	const CairoColor *base = &colors->base[params->state_type];
	const CairoColor *border;
	
	if (params->focus) border = &colors->spot[1];
	else border = &colors->shade[params->disabled ? 4 : 6];

	cairo_translate (cr, x+0.5, y+0.5);
	cairo_set_line_width (cr, 1.0);
	
	/* Fill the background (shouldn't have to) */
	cairo_rectangle (cr, -0.5, -0.5, width+1, height+1);
	cairo_set_source_rgb (cr, params->parentbg.r, 
	                          params->parentbg.g, 
	                          params->parentbg.b);
	cairo_fill (cr);

	/* Fill the entry's base color (why isn't is large enough by default?) */
	cairo_rectangle (cr, 1.5, 2.5, width-4, height-5);
	cairo_set_source_rgb (cr, base->r, 
	                          base->g, 
	                          base->b);
	cairo_fill (cr);
	

	// Draw the inner shadow 
	if (params->focus)
	{
		cairo_rectangle (cr, 2, 2, width-5, height-5);
		cairo_set_source_rgb (cr, colors->spot[1].r, colors->spot[1].g, colors->spot[1].b);
		cairo_stroke (cr);
	}
	else
	{
		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, params->disabled ? 0.05 : 0.1);
		cairo_move_to (cr, 2, height-3);
		cairo_line_to (cr, 2, 2);
		cairo_line_to (cr, width-3, 2);
		cairo_stroke (cr);
	}

	if (params->focus) border = &colors->spot[1];
	else border = &colors->shade[params->disabled ? 4 : 6];


	// Draw the border 
	if (params->focus)
	{
		cairo_set_line_width (cr, 3.5);
		aqualooks_rounded_rectangle (cr, 1.5, 1.5, width-3, height-3, RADIUS, params->corners);
		cairo_set_source_rgb (cr, colors->spot[0].r, colors->spot[0].g, colors->spot[0].b);
		cairo_stroke (cr);
	}
	else
	{
		cairo_set_line_width (cr, 1.2);
		cairo_set_source_rgb (cr, border->r, border->g, border->b);
		cairo_rectangle (cr, 1, 1, width-3, height-3);
		cairo_stroke (cr);

		cairo_set_line_width (cr, 1.2);
		cairo_set_source_rgb (cr, border->r, border->g, border->b);
		cairo_rectangle (cr, 1, 2, width-3, height-4);
		cairo_stroke (cr);
	}
}

void
aqualooks_draw_spinbutton (cairo_t *cr,
                            const AqualooksColors *colors,
                            const WidgetParameters *params,
                            int x, int y, int width, int height)
{
	ShadowParameters shadow = { CL_CORNER_ALL, CL_SHADOW_IN };
	
	aqualooks_draw_button (cr, colors, params, &shadow, x, y, width, height, 5);
	

	cairo_set_line_width (cr, 1);
	
	/* Seperator */
	cairo_move_to (cr, params->xthickness      , 0.5 + (height/2));
	cairo_line_to (cr, width-params->xthickness, 0.5 + (height/2));
	cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.3);
	cairo_stroke (cr);

	cairo_move_to (cr, params->xthickness,       0.5 + (height/2)+1);
	cairo_line_to (cr, width-params->xthickness, 0.5 + (height/2)+1);
	cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.7);
	cairo_stroke (cr);	
}

void
aqualooks_draw_spinbutton_down (cairo_t *cr,
                                 const AqualooksColors *colors,
                                 const WidgetParameters *params,
                                 int x, int y, int width, int height)
{
	cairo_pattern_t *pattern;
	
	cairo_translate (cr, x+1, y+1);
	
	aqualooks_rounded_rectangle (cr, 1, 1, width-4, height-4, 3, params->corners);
	
	cairo_set_source_rgb (cr, colors->bg[params->state_type].r,
	                          colors->bg[params->state_type].g, 
	                          colors->bg[params->state_type].b);
	
	cairo_fill_preserve (cr);
	
	pattern = cairo_pattern_create_linear (0, 0, 0, height);
	cairo_pattern_add_color_stop_rgba (pattern, 0.0, 0.0, 0.0, 0.0, 0.3);
	cairo_pattern_add_color_stop_rgba (pattern, 1.0, 0.0, 0.0, 0.0, 0.0);
	
	cairo_set_source (cr, pattern);
	cairo_fill (cr);
	
	cairo_pattern_destroy (pattern);
}

static void
aqualooks_scale_draw_gradient (cairo_t *cr,
                                const CairoColor *c1,
                                const CairoColor *c2,
                                const CairoColor *c3,
                                int x, int y, int width, int height,
                                boolean horizontal)
{
	cairo_pattern_t *pattern;

	pattern = cairo_pattern_create_linear (0, 0, horizontal ? 0 :  width, horizontal ? height : 0);
	cairo_pattern_add_color_stop_rgb (pattern, 0.0, c1->r, c1->g, c1->b);
	cairo_pattern_add_color_stop_rgb (pattern, 1.0, c2->r, c2->g, c2->b);

	cairo_rectangle (cr, x+0.5, y+0.5, width-1, height-1);	
	cairo_set_source (cr, pattern);
	cairo_fill (cr);
	cairo_pattern_destroy (pattern);
	
	cairo_rectangle (cr, x, y, width, height);	
	cairo_set_source_rgb (cr, c3->r, c3->g, c3->b);
	cairo_stroke (cr);
}

#define TROUGH_SIZE 6
void
aqualooks_draw_scale_trough (cairo_t *cr,
                              const AqualooksColors *colors,
                              const WidgetParameters *params,
                              const SliderParameters *slider,
                              int x, int y, int width, int height)
{
	int     fill_x, fill_y, fill_width, fill_height; /* Fill x,y,w,h */
	int     trough_width, trough_height;
	double  translate_x, translate_y;
	int     fill_size = slider->fill_size;

	if (slider->horizontal)
	{
		if (fill_size > width-3)
			fill_size = width-3;

		fill_x        = slider->inverted ? width - fill_size - 3 : 0;
		fill_y        = 0;
		fill_width    = fill_size;			
		fill_height   = TROUGH_SIZE-2;
		
		trough_width  = width-3;
		trough_height = TROUGH_SIZE-2;
		
		translate_x   = x + 0.5;
		translate_y   = y + 0.5 + (height/2) - (TROUGH_SIZE/2);
	}
	else
	{
		if (fill_size > height-3)
			fill_size = height-3;

		fill_x        = 0;
		fill_y        = slider->inverted ? height - fill_size - 3 : 0;
		fill_width    = TROUGH_SIZE-2;
		fill_height   = fill_size;			
		
		trough_width  = TROUGH_SIZE-2;
		trough_height = height-3;
		
		translate_x   = x + 0.5 + (width/2) - (TROUGH_SIZE/2);
		translate_y  = y + 0.5;
	}

	cairo_set_line_width (cr, 1.0);
	cairo_translate (cr, translate_x, translate_y);
	
	aqualooks_draw_inset (cr, trough_width+2, trough_height+2, 0, 0);
	
	cairo_translate (cr, 1, 1);
	aqualooks_scale_draw_gradient (cr, &colors->shade[5], /* top */
	                                    &colors->shade[3], /* bottom */
	                                    &colors->shade[6], /* border */
	                                    0, 0, trough_width, trough_height,
	                                    slider->horizontal);

}

void
aqualooks_draw_slider_button (cairo_t *cr,
                               const AqualooksColors *colors,
                               const WidgetParameters *params,
                               const SliderParameters *slider,
                               int x, int y, int width, int height)
{
	const CairoColor *border = &colors->shade[6];
	CairoColor        top, bottom;
	cairo_pattern_t *pattern;
	ShadowParameters shadow = { CL_CORNER_ALL, CL_SHADOW_IN };
	double xoffset = 0, yoffset = 0;

	cairo_set_line_width (cr, 1.0);
	
	if (slider->horizontal)
	{
		rotate_mirror_translate (cr, 0, x, y, FALSE, FALSE);
	}
	else
	{
		rotate_mirror_translate (cr, M_PI/2, x, y, FALSE, FALSE);
	}
	
	if (params->xthickness == 3)
		xoffset = 1;
	if (params->ythickness == 3)
		yoffset = 1;

	aqualooks_draw_shadow (cr, width-1, height-1);
	aqualooks_draw_button (cr, colors, params, &shadow, xoffset, yoffset, width-(xoffset*2), height-(yoffset*2), 7);

}

static cairo_surface_t*
aqualooks_progressbar_create_cell (int size, const CairoColor *color, gboolean fill)
{
	cairo_t *cr;
	cairo_surface_t *cell;
	cairo_pattern_t *pt;
	CairoColor a, b, c, d, e, f, g, h;


		ul_shade (color, &a, 1.12);
		ul_shade (color, &b, 1.00);
		ul_shade (color, &c, 0.96);
		ul_shade (color, &d, 0.92);
		ul_shade (color, &e, 0.74);
		ul_shade (color, &f, 0.88 );
		ul_shade (color, &g, 0.96 );
		ul_shade (color, &h, 1.24 );


	cell = cairo_image_surface_create (CAIRO_FORMAT_RGB24, size, size);
	cr = cairo_create (cell);

	pt = cairo_pattern_create_linear (0, 0, 0, size);

	cairo_pattern_add_color_stop_rgb (pt, 0.0,  a.r, a.g, a.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.2,  b.r, b.g, b.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.3,  c.r, c.g, c.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.45, d.r, d.g, d.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.45, e.r, e.g, e.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.65, f.r, f.g, f.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.85, g.r, g.g, g.b);
	cairo_pattern_add_color_stop_rgb (pt, 1.0,  h.r, h.g, h.b);

	cairo_set_source (cr, pt);
	cairo_rectangle (cr, 0, 0, size, size);
	cairo_fill (cr);

	cairo_set_line_width (cr, 0);

	cairo_pattern_destroy (pt);
	cairo_destroy (cr);

	return cell;
}

void
aqualooks_draw_progressbar_trough (cairo_t *cr,
                                    const AqualooksColors      *colors,
                                    const WidgetParameters      *params,
                                    const ProgressBarParameters *progressbar,
                                    int x, int y, int width, int height)
{
	boolean           is_horizontal = progressbar->orientation < 2;
	const CairoColor *border = &colors->shade[7];
	cairo_pattern_t  *pt;
	int               size;
	
	cairo_set_line_width (cr, 2.0);
	
	/* Fill with bg color */
	cairo_set_source_rgb (cr, colors->bg[params->state_type].r,
	                          colors->bg[params->state_type].g,
	                          colors->bg[params->state_type].b);
	
	cairo_rectangle (cr, x, y, width, height);	
	cairo_fill (cr);

	if (is_horizontal)
		size = height - params->ythickness*2;
	else
		size = width - params->xthickness*2;

	/* Draw the border before anything else... */
	cairo_rectangle (cr, x + params->xthickness - 0.5,
	                                  y + params->ythickness - 0.5,
	                                  width - params->xthickness - 1,
	                                  height - params->ythickness - 1);
	cairo_set_source_rgb (cr, colors->shade[7].r, colors->shade[7].g, colors->shade[7].b);
	cairo_stroke (cr);

	/* Create cell image surface, create a pattern from it */
	cairo_surface_t *cell = aqualooks_progressbar_create_cell (size, &colors->bg[params->state_type], FALSE);
	pt = cairo_pattern_create_for_surface (cell);
	cairo_pattern_set_extend (pt, CAIRO_EXTEND_REPEAT);

	cairo_matrix_t m;
	cairo_matrix_init_translate (&m, -1, -1);
	cairo_pattern_set_matrix (pt, &m);

	if (is_horizontal)
	{
		rotate_mirror_translate (cr, 0, x, y, FALSE, FALSE);
		cairo_matrix_init_translate (&m, x-1, 0);
	}
	else
	{
		int tmp = height;
		height  = width;
		width   = tmp;

		rotate_mirror_translate (cr, M_PI/2, x, y, FALSE, FALSE);
		cairo_matrix_init_translate (&m, y-1, 0);
	}

	cairo_set_source (cr, pt);
	cairo_pattern_destroy (pt);
	cairo_rectangle (cr, params->xthickness, params->ythickness, width-params->xthickness*2, height-params->ythickness*2);
	cairo_fill (cr);
	cairo_surface_destroy (cell);

	if (params->xthickness > 1 && params->ythickness > 1)
	{
		cairo_translate (cr, -0.5, -0.5);
		aqualooks_draw_shadow (cr, width, height);
	}
}

void
aqualooks_draw_progressbar_fill (cairo_t *cr,
                                  const AqualooksColors *colors,
                                  const WidgetParameters *params,
                                  const ProgressBarParameters *progressbar,
                                  int x, int y, int width, int height,
                                  gint offset)
{
	boolean          is_horizontal = progressbar->orientation < 2;
	double           tile_pos = 0;
	double           stroke_width;
	int			x_step;

	cairo_pattern_t *pt;
	CairoColor       shade1;
	cairo_matrix_t m;
	
	cairo_rectangle (cr, x, y, width, height);
	cairo_clip (cr);
	cairo_new_path (cr);

	if (is_horizontal)
	{
		rotate_mirror_translate (cr, 0, x, y, FALSE, FALSE);
		cairo_matrix_init_translate (&m, x-1, 0);
	}
	else
	{
		int tmp = height;
		height  = width;
		width   = tmp;

		rotate_mirror_translate (cr, M_PI/2, x, y, FALSE, FALSE);
		cairo_matrix_init_translate (&m, y-1, 0);
	}

	/* Create cell image surface, create a pattern from it */
	cairo_surface_t *cell = aqualooks_progressbar_create_cell (height, &colors->spot[1], TRUE);
	pt = cairo_pattern_create_for_surface (cell);
	cairo_pattern_set_extend (pt, CAIRO_EXTEND_REPEAT);
	cairo_pattern_set_matrix (pt, &m);
	cairo_set_source (cr, pt);
	cairo_pattern_destroy (pt);
	
	cairo_rectangle (cr, 0, 0, width, height);
	cairo_fill (cr);

	cairo_surface_destroy (cell);
}

void
aqualooks_draw_optionmenu (cairo_t *cr,
                            const AqualooksColors *colors,
                            const WidgetParameters *params,
                            const OptionMenuParameters *optionmenu,
                            int x, int y, int width, int height)
{
	ShadowParameters shadow = { CL_CORNER_ALL, CL_SHADOW_NONE };

	int offset = params->ythickness + 1;
	
	aqualooks_draw_button (cr, colors, params, &shadow, x, y, width, height, 5);
	
	cairo_set_line_width  (cr, 1.0);
	cairo_translate       (cr, optionmenu->linepos + 0.5, 0.5);
	
	cairo_move_to         (cr, 0.0, offset);
	cairo_line_to         (cr, 0.0, height - offset - params->ythickness + 1);
	cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.2);
	cairo_stroke          (cr);
	
	cairo_move_to         (cr, 1.0, offset);
	cairo_line_to         (cr, 1.0, height - offset - params->ythickness + 1);
	cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.8);
	cairo_stroke          (cr);
}

static void
ubuntulooks_menubar_gradient (cairo_t *cr,
                              double x, double y, int width, int height,
                              CairoColor *color, boolean fill, boolean disabled)
{
	CairoColor a, b, c, d, e,f,g,h,i,j,k;
	cairo_pattern_t *pt;

	if (fill)
	{
		ul_shade (color, &a, 1.12);
		ul_shade (color, &b, 1.00);
		ul_shade (color, &c, 0.96);
		ul_shade (color, &d, 0.92);
		ul_shade (color, &e, 0.74);
		ul_shade (color, &f, 0.88 );
		ul_shade (color, &g, 0.96 );
		ul_shade (color, &h, 1.16 );


	} else {
		ul_shade (color, &a, disabled? 1.03 : 1.15);
		ul_shade (color, &b, disabled? 1.01 : 1.02);
		ul_shade (color, &c, disabled? 0.99 : 0.96);
		ul_shade (color, &d, 0.92 );
		ul_shade (color, &e, disabled? 1.01 : 0.74);
		ul_shade (color, &f, disabled? 1.01 : 0.88);
		ul_shade (color, &g, disabled? 1.01 : 0.96);
		ul_shade (color, &h, disabled? 1.01 : 1.16);

	}

	pt = cairo_pattern_create_linear (x, y, x, y+height);
	cairo_pattern_add_color_stop_rgb (pt, 0.0,  a.r, a.g, a.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.2,  b.r, b.g, b.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.3,  c.r, c.g, c.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.45, d.r, d.g, d.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.45, e.r, e.g, e.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.65, f.r, f.g, f.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.80, g.r, g.g, g.b);
	cairo_pattern_add_color_stop_rgb (pt, 1.0,  h.r, h.g, h.b);

	cairo_set_source (cr, pt);
	cairo_rectangle (cr, x, y, width, height);
	cairo_fill (cr);
	
	cairo_pattern_destroy (pt);
}

void
aqualooks_draw_menubar (cairo_t *cr,
                         const AqualooksColors *colors,
                         const WidgetParameters *params, boolean fixed,
                         int x, int y, int width, int height)
{
	CairoColor lower;
	cairo_pattern_t *pattern;
	CairoColor fill         = colors->bg[params->state_type];
	
	ul_shade (&colors->bg[0], &lower, 0.95);
	
	cairo_translate (cr, x, y);
	cairo_rectangle (cr, x, y, width, height);
	
	// Draw the gradient
	if (fixed)
	{
		pattern = cairo_pattern_create_linear (x, y, x, height);
		cairo_pattern_add_color_stop_rgb (pattern, 0.0, colors->bg[0].r,
	                                                colors->bg[0].g,
	                                                colors->bg[0].b);
		cairo_pattern_add_color_stop_rgb (pattern, 1.0, lower.r,
	                                                lower.g,
	                                                lower.b);
		cairo_set_source      (cr, pattern);
		cairo_fill            (cr);
		cairo_pattern_destroy (pattern);
	}
	else
	{

		if (params->active && params->prelight)
		{
			fill = colors->bg[CL_STATE_ACTIVE];
			ul_shade (&fill, &fill, 1.05);
		}
		ubuntulooks_menubar_gradient (cr,0,0,width,height, &fill, FALSE, params->disabled);
	}
	
	/* Draw bottom line */
	cairo_set_line_width  (cr, 1.0);
	cairo_move_to         (cr, x, y+height-0.5);
	cairo_line_to         (cr, width, height-0.5);
	cairo_set_source_rgb  (cr, colors->shade[3].r,
	                           colors->shade[3].g,
	                           colors->shade[3].b);
	cairo_stroke          (cr);
}

static void
aqualooks_get_frame_gap_clip (int x, int y, int width, int height, 
                               FrameParameters     *frame,
                               AqualooksRectangle *bevel,
                               AqualooksRectangle *border)
{
	if (frame->gap_side == CL_GAP_TOP)
	{
		AQUALOOKS_RECTANGLE_SET ((*bevel),  1.5 + frame->gap_x,  -0.5,
											 frame->gap_width - 3, 2.0);
		AQUALOOKS_RECTANGLE_SET ((*border), 0.5 + frame->gap_x,  -0.5,
											 frame->gap_width - 2, 2.0);
	}
	else if (frame->gap_side == CL_GAP_BOTTOM)
	{
		AQUALOOKS_RECTANGLE_SET ((*bevel),  1.5 + frame->gap_x,  height - 2.5,
											 frame->gap_width - 3, 2.0);
		AQUALOOKS_RECTANGLE_SET ((*border), 0.5 + frame->gap_x,  height - 1.5,
											 frame->gap_width - 2, 2.0);		
	}
	else if (frame->gap_side == CL_GAP_LEFT)
	{
		AQUALOOKS_RECTANGLE_SET ((*bevel),  -0.5, 1.5 + frame->gap_x,
											 2.0, frame->gap_width - 3);
		AQUALOOKS_RECTANGLE_SET ((*border), -0.5, 0.5 + frame->gap_x,
											 1.0, frame->gap_width - 2);			
	}
	else if (frame->gap_side == CL_GAP_RIGHT)
	{
		AQUALOOKS_RECTANGLE_SET ((*bevel),  width - 2.5, 1.5 + frame->gap_x,
											 2.0, frame->gap_width - 3);
		AQUALOOKS_RECTANGLE_SET ((*border), width - 1.5, 0.5 + frame->gap_x,
											 1.0, frame->gap_width - 2);			
	}
}

void
aqualooks_draw_frame            (cairo_t *cr,
                                  const AqualooksColors     *colors,
                                  const WidgetParameters     *params,
                                  const FrameParameters      *frame,
                                  int x, int y, int width, int height)
{
	const float RADIUS = 4.5;
	CairoColor *border = frame->border;
	AqualooksRectangle bevel_clip;
	AqualooksRectangle frame_clip;
	
	if (frame->shadow == CL_SHADOW_NONE)
		return;
	
	if (frame->gap_x != -1)
		aqualooks_get_frame_gap_clip (x, y, width, height,
		                               (FrameParameters*)frame,
		                               &bevel_clip, &frame_clip);
	
	cairo_set_line_width (cr, 1.0);
	cairo_translate      (cr, x+0.5, y+0.5);
	
	/* Set clip for the bevel */
	if (frame->gap_x != -1)
	{
		/* Set clip for gap */
		cairo_set_fill_rule  (cr, CAIRO_FILL_RULE_EVEN_ODD);
		aqualooks_rounded_rectangle (cr, -0.5, -0.5, width, height, RADIUS, params->corners);
		aqualooks_rounded_rectangle (cr, bevel_clip.x, bevel_clip.y, bevel_clip.width, bevel_clip.height, RADIUS, params->corners);
		cairo_clip           (cr);
		cairo_new_path       (cr);
	}
	
	/* Draw the bevel */
	if (frame->shadow == CL_SHADOW_ETCHED_IN || frame->shadow == CL_SHADOW_ETCHED_OUT)
	{
		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.8 );
		if (frame->shadow == CL_SHADOW_ETCHED_IN)
			aqualooks_rounded_rectangle (cr, 1, 1, width-2, height-2, RADIUS, params->corners);
		else
			aqualooks_rounded_rectangle (cr, 0, 0, width-2, height-2, RADIUS, params->corners);
		cairo_stroke          (cr);
	}
	else if (frame->shadow != CL_SHADOW_NONE)
	{
		ShadowParameters shadow;
		shadow.corners = params->corners;
		shadow.shadow  = frame->shadow;
		aqualooks_draw_highlight_and_ul_shade (cr, &shadow, width, height, RADIUS);
	}
	
	/* Set clip for the frame */
	cairo_reset_clip     (cr);
	if (frame->gap_x != -1)
	{
		/* Set clip for gap */
		cairo_set_fill_rule  (cr, CAIRO_FILL_RULE_EVEN_ODD);
		aqualooks_rounded_rectangle (cr, -0.5, -0.5, width, height, RADIUS, params->corners);
		aqualooks_rounded_rectangle (cr, frame_clip.x, frame_clip.y, frame_clip.width, frame_clip.height, RADIUS, params->corners);
		cairo_clip           (cr);
		cairo_new_path       (cr);
	}

	/* Draw frame */
	if (frame->shadow == CL_SHADOW_ETCHED_IN || frame->shadow == CL_SHADOW_ETCHED_OUT)
	{
		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.2 );
		if (frame->shadow == CL_SHADOW_ETCHED_IN)
			aqualooks_rounded_rectangle (cr, 0, 0, width-2, height-2, RADIUS, params->corners);
		else
			aqualooks_rounded_rectangle (cr, 1, 1, width-2, height-2, RADIUS, params->corners);
	}
	else
	{
		cairo_set_source_rgb (cr, border->r, border->g, border->b );
		aqualooks_rounded_rectangle (cr, 0, 0, width-1, height-1, RADIUS, params->corners);
	}
	cairo_stroke         (cr);
}

void aqualooks_draw_tab_gradients(cairo_t *cr,
                                  double x, double y, int width, int height,
                                  CairoColor *color, boolean fill, boolean disabled, double radius, uint8 corners)
{
 
	CairoColor a, b, c, d, e,f,g,h,i,j,k;
	cairo_pattern_t *pt;

	if (fill)
	{
		ul_shade (color, &a, 1.12);
		ul_shade (color, &b, 1.00);
		ul_shade (color, &c, 0.96);
		ul_shade (color, &d, 0.92);
		ul_shade (color, &e, 0.74);
		ul_shade (color, &f, 0.88 );
		ul_shade (color, &g, 0.96 );
		ul_shade (color, &h, 1.36 );


	} else {
		ul_shade (color, &a, disabled? 1.03 : 1.15);
		ul_shade (color, &b, disabled? 1.01 : 1.02);
		ul_shade (color, &c, disabled? 0.99 : 0.96);
		ul_shade (color, &d, 0.92 );
		ul_shade (color, &e, disabled? 1.01 : 0.74);
		ul_shade (color, &f, disabled? 1.01 : 0.88);
		ul_shade (color, &g, disabled? 1.01 : 0.96);
		ul_shade (color, &h, disabled? 1.01 : 1.36);

	}

	pt = cairo_pattern_create_linear (x, y, x, y+height);
	cairo_pattern_add_color_stop_rgb (pt, 0.0,  a.r, a.g, a.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.2,  b.r, b.g, b.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.3,  c.r, c.g, c.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.45, d.r, d.g, d.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.45, e.r, e.g, e.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.65, f.r, f.g, f.b);
	cairo_pattern_add_color_stop_rgb (pt, 0.80, g.r, g.g, g.b);
	cairo_pattern_add_color_stop_rgb (pt, 1.0,  h.r, h.g, h.b);


	cairo_set_source (cr, pt);
	aqualooks_rounded_rectangle (cr, x, y, width, height, radius, corners);
	cairo_fill (cr);
	
	cairo_pattern_destroy (pt);

}

void
aqualooks_draw_tab (cairo_t *cr,
                     const AqualooksColors *colors,
                     const WidgetParameters *params,
                     const TabParameters    *tab,
                     int x, int y, int width, int height)
{
	const float RADIUS = 2.5;
	
	const CairoColor    *border1       = &colors->shade[6];
	const CairoColor    *stripe_fill   = &colors->spot[1];
	const CairoColor    *stripe_border = &colors->spot[2];
	CairoColor fill     		   = colors->bg[params->state_type];

	cairo_pattern_t     *pattern;
	
	int                  corners;
	double               strip_size;

	
	// Translate and set line width 	
	cairo_set_line_width (cr, 0.5);
	cairo_translate      (cr, x+0.5, y+0.5);

	x = 0;
	y = 0;


	// Make the tabs slightly bigger than they should be, to create a gap 
	// And calculate the strip size too, while you're at it 
	if (tab->gap_side == CL_GAP_TOP || tab->gap_side == CL_GAP_BOTTOM)
	{
		height += RADIUS;
	 	strip_size = 1.0 / height * 1.0;
		
		if (tab->gap_side == CL_GAP_TOP)
		{
			cairo_translate (cr, 1.0, 1.0); // gap at the other side 
		}

	}
	else
	{
		width += RADIUS;
	 	strip_size = 1.0 / width * 1.0;
		
		if (tab->gap_side == CL_GAP_LEFT) 
		{
			cairo_translate (cr, 1.0, 1.0); // gap at the other side 
		}
	}
	
	corners = CL_CORNER_ALL;
	

	// Draw highlight 

	if (!params->active)
	{
		ShadowParameters shadow;
		
		shadow.shadow  = CL_SHADOW_OUT;
		shadow.corners = corners;
		
		aqualooks_draw_highlight_and_ul_shade (cr, &shadow, width  -1, height -1, RADIUS);
	}


	if (!params->active)
	{
		fill = colors->bg[CL_STATE_SELECTED];
		ul_shade (&fill, &fill, 1.05);
	}

	if (params->active)
	{
		cairo_set_line_width (cr, 1);
		cairo_set_source_rgba (cr, x+1, y+1, 1, 0.4);

			
		
		if (tab->gap_side == CL_GAP_TOP || tab->gap_side == CL_GAP_BOTTOM)
			aqualooks_rounded_rectangle (cr, x+0, y+0, width-1, height-1, RADIUS, corners);
		else
			aqualooks_rounded_rectangle (cr, x+0, y+0, width-1, height-1, RADIUS, corners);
		
		aqualooks_rounded_rectangle (cr, x+0, y+0, width-1, height-1, RADIUS, corners);
		
		// bling overlay 

		pattern = cairo_pattern_create_linear (x+0, y+0, x+0, height);
		cairo_pattern_add_color_stop_rgba (pattern, 0.0, 1.0, 1.0, 1.0, 0.6);
		cairo_pattern_add_color_stop_rgba (pattern, 0.4, 1.0, 1.0, 1.0, 0.3);

		cairo_set_source (cr, pattern);
		cairo_fill (cr);
		cairo_pattern_destroy (pattern);

		// Draw gradient
		aqualooks_draw_tab_gradients (cr,x+0, y+0, width-1, height-1, &fill, FALSE, params->disabled, RADIUS, params->corners);
	}
	else
	{
		// Draw shade 
		pattern = cairo_pattern_create_linear ( tab->gap_side == CL_GAP_LEFT   ? width-2  : 0,
		                                        tab->gap_side == CL_GAP_TOP    ? height-2 : 0,
		                                        tab->gap_side == CL_GAP_RIGHT  ? width    : 0,
		                                        tab->gap_side == CL_GAP_BOTTOM ? height   : 0 );
	
		aqualooks_rounded_rectangle (cr, x+0, y+0, width-1, height-1, RADIUS, corners);
		
		cairo_pattern_add_color_stop_rgb  (pattern, 0.0, stripe_fill->r, stripe_fill->g, stripe_fill->b);
		cairo_pattern_add_color_stop_rgb  (pattern, strip_size, stripe_fill->r, stripe_fill->g, stripe_fill->b);

		cairo_set_source (cr, pattern);
		cairo_fill (cr);
		cairo_pattern_destroy (pattern);
		// Draw gradient
		aqualooks_draw_tab_gradients (cr,x+0, y+0, width-1, height-1, &fill, FALSE, params->disabled, RADIUS, params->corners);

	}



	cairo_set_line_width (cr, 0.5);
	aqualooks_rounded_rectangle (cr, x+0, y+0, width-1, height-1, RADIUS, corners);
	

	if (params->active)
	{
		cairo_set_source_rgb  (cr, border1->r, border1->g, border1->b);	
		cairo_stroke (cr);
	}
	else
	{
		pattern = cairo_pattern_create_linear ( tab->gap_side == CL_GAP_LEFT   ? width-1  : 2,
		                                        tab->gap_side == CL_GAP_TOP    ? height-1 : 2,
		                                        tab->gap_side == CL_GAP_RIGHT  ? width    : 2,
		                                        tab->gap_side == CL_GAP_BOTTOM ? height   : 2 );
		
		cairo_pattern_add_color_stop_rgb (pattern, 0.0, stripe_border->r, stripe_border->g, stripe_border->b);
		cairo_pattern_add_color_stop_rgb (pattern, strip_size, stripe_border->r, stripe_border->g, stripe_border->b);
		cairo_pattern_add_color_stop_rgb (pattern, strip_size, border1->r, border1->g, border1->b);
		cairo_pattern_add_color_stop_rgb (pattern, 1.0, border1->r, border1->g, border1->b);
		cairo_set_source (cr, pattern);
		cairo_stroke (cr);
		cairo_pattern_destroy (pattern);
	}


}

void
aqualooks_draw_separator (cairo_t *cr,
                           const AqualooksColors     *colors,
                           const WidgetParameters     *widget,
                           const SeparatorParameters  *separator,
                           int x, int y, int width, int height)
{
	if (separator->horizontal)
	{
		cairo_set_line_width  (cr, 1.0);
		cairo_translate       (cr, x, y+0.5);
		
		cairo_move_to         (cr, 0.0,     0.0);
		cairo_line_to         (cr, width+1, 0.0);
		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.2);
		cairo_stroke          (cr);
		
		cairo_move_to         (cr, 0.0,   1.0);
		cairo_line_to         (cr, width, 1.0);
		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.8);
		cairo_stroke          (cr);				
	}
	else
	{
		cairo_set_line_width  (cr, 1.0);
		cairo_translate       (cr, x+0.5, y);
		
		cairo_move_to         (cr, 0.0, 0.0);
		cairo_line_to         (cr, 0.0, height);
		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.2);
		cairo_stroke          (cr);
		
		cairo_move_to         (cr, 1.0, 0.0);
		cairo_line_to         (cr, 1.0, height);
		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.8);
		cairo_stroke          (cr);		
	}
}

void
aqualooks_draw_list_view_header (cairo_t *cr,
                                  const AqualooksColors          *colors,
                                  const WidgetParameters          *widget,
                                  const ListViewHeaderParameters  *header,
                                  int x, int y, int width, int height)
{
	CairoColor      *border = (CairoColor*)&colors->shade[3];
	CairoColor fill         = colors->bg[widget->state_type];
	CairoColor hilight;
	ul_shade (border, &hilight, 1.5);
	
	cairo_translate (cr, x, y);
	cairo_set_line_width (cr, 1.0);
	
	/* Draw highlight */
	if (header->order == CL_ORDER_FIRST)
	{
		cairo_move_to (cr, 0.5, height-1);
		cairo_line_to (cr, 0.5, 0.5);
	}
	else
		cairo_move_to (cr, 0.0, 0.5);
	
	cairo_line_to (cr, width, 0.5);
	
	cairo_set_source_rgb (cr, hilight.r, hilight.g, hilight.b);
	cairo_stroke (cr);

	if (widget->active && widget->prelight)
	{
		fill = colors->bg[CL_STATE_ACTIVE];
		ul_shade (&fill, &fill, 1.05);
	}
	

	aqualooks_draw_button_gradient (cr,0,0,width,height, &fill, FALSE, widget->disabled, 1, widget->corners);
	

	/* Draw bottom border */
	cairo_move_to (cr, 0.0, height-0.5);
	cairo_line_to (cr, width, height-0.5);
	cairo_set_source_rgb (cr, border->r,
	                          border->g,
	                          border->b);
	cairo_stroke (cr);
	/* Draw resize grip */
	if (header->order != CL_ORDER_LAST || header->resizable)
	{
		SeparatorParameters separator;
		separator.horizontal = FALSE;
		
		aqualooks_draw_separator (cr, colors, widget, &separator,
		                           width-1.5, 4.0, 2, height-8.0);
	}
}

/* We can't draw transparent things here, since it will be called on the same
 * surface multiple times, when placed on a handlebox_bin or dockitem_bin */
void
aqualooks_draw_toolbar (cairo_t *cr,
                         const AqualooksColors          *colors,
                         const WidgetParameters          *widget,
                         int x, int y, int width, int height, boolean gradient)
{

	/* Aqualooks - toolbar gradient */
	if (gradient)
	{
	cairo_rectangle (cr, x, y, width, height);
	cairo_move_to (cr,x,y);
	cairo_pattern_t *pattern;
	CairoColor lower;
	ul_shade (&colors->bg[0], &lower, 0.85);
	pattern = cairo_pattern_create_linear (x, y, x, height);
	cairo_pattern_add_color_stop_rgb (pattern, 0.0, colors->bg[0].r, colors->bg[0].g, colors->bg[0].b);
	cairo_pattern_add_color_stop_rgb (pattern, 1.0, lower.r, lower.g, lower.b);
	cairo_set_source (cr, pattern);
	cairo_fill (cr);
	cairo_pattern_destroy 	(pattern);
	}
	/* -- End of toolbar gradient --- */

	const CairoColor *light = &colors->shade[0];
	const CairoColor *dark  = &colors->shade[3];
	
	cairo_set_line_width (cr, 1.0);

	cairo_translate       (cr, x, y);

	/* Draw shadow */
	cairo_move_to         (cr, x, height-0.5);
	cairo_line_to         (cr, width-1, height-0.5);
	cairo_set_source_rgb  (cr, dark->r, dark->g, dark->b);
	cairo_stroke          (cr);
}

void
aqualooks_draw_menuitem (cairo_t *cr,
                          const AqualooksColors          *colors,
                          const WidgetParameters          *widget,
                          int x, int y, int width, int height)
{
	const CairoColor *fill = &colors->spot[1];
	CairoColor        fill_shade;
	CairoColor        border_top, border_bottom;
	cairo_pattern_t  *pattern;
	
	ul_shade (fill, &fill_shade, 0.85);
	
	ul_shade (fill, &border_top, 0.9);
	ul_shade (fill, &border_bottom, 0.8);
	
	cairo_translate      (cr, x+0.5, y+0.5);
	cairo_set_line_width (cr, 1.0);
	
	cairo_rectangle (cr, 0, 0, width-1, height-1);
	
	pattern = cairo_pattern_create_linear (0, 0, 0, height);
	cairo_pattern_add_color_stop_rgb (pattern, 0,   fill->r, fill->g, fill->b);
	cairo_pattern_add_color_stop_rgb (pattern, 1.0, fill_shade.r, fill_shade.g, fill_shade.b);
	
	cairo_set_source (cr, pattern);
	cairo_fill (cr);
	cairo_pattern_destroy (pattern);


	aqualooks_rounded_rectangle (cr, 0, 0, width-1, height-1, 1.25, widget->corners);
	
	pattern = cairo_pattern_create_linear (0, 0, 0, height);
	cairo_pattern_add_color_stop_rgb (pattern, 0,   border_top.r, border_top.g, border_top.b);
	cairo_pattern_add_color_stop_rgb (pattern, 1.0, border_bottom.r, border_bottom.g, border_bottom.b);
	
	cairo_set_source (cr, pattern);
	cairo_stroke	         (cr);
	cairo_pattern_destroy (pattern);
}

void
aqualooks_draw_scrollbar_trough (cairo_t *cr,
                                  const AqualooksColors            *colors,
                                  const WidgetParameters           *params,
                                  const ScrollBarParameters        *scrollbar,
                                  int x, int y, int width, int height)
{



	const CairoColor *border = &colors->shade[ params->disabled ? 4 : 6];
	const CairoColor *bg     = &colors->bg[params->state_type];
	CairoColor  fill = (CairoColor)colors->shade[0];
	CairoColor	shade1, shade2, shade3, shade4;
	cairo_pattern_t *pattern;
	ShadowParameters shadow;
	CairoColor        bg_shade;
	CairoColor        center;


	if (scrollbar->horizontal)
	{
		cairo_translate (cr, x, y);	
	}
	else
	{
		int tmp = height;
		rotate_mirror_translate (cr, M_PI/2, x, y, FALSE, FALSE);
		height = width;
		width = tmp;
	}


	//cairo_translate (cr, x, y);

	cairo_set_line_width (cr, 1);
	                                     
	if (params->prelight) {
		ul_shade (&fill, &fill, 1.1);
	}
		
	cairo_set_line_width (cr, 1);
	
	ul_shade (&fill, &shade1, 1.1);
	ul_shade (&fill, &shade2, 1.05);
	ul_shade (&fill, &shade3, 0.85);
	ul_shade (&fill, &shade4, 1.0);


	pattern = cairo_pattern_create_linear (1, 1, 1, height-4);
	cairo_pattern_add_color_stop_rgb (pattern, 0,   shade1.r, shade1.g, shade1.b);
	cairo_pattern_add_color_stop_rgb (pattern, 0.5, shade2.r, shade2.g, shade2.b);
	cairo_pattern_add_color_stop_rgb (pattern, 1,   shade4.r, shade4.g, shade4.b);	
	cairo_pattern_add_color_stop_rgb (pattern, 0.5, shade3.r,  shade3.g,  shade3.b);
	cairo_rectangle (cr, 1, 1, width-2, height-2);

	cairo_set_source (cr, pattern);
	cairo_fill (cr);
	cairo_pattern_destroy (pattern);
	
	cairo_rectangle (cr, 0, 0, width, height);
	cairo_set_source_rgba (cr, 1, 1, 1, 0.6);
	cairo_stroke (cr);


	bg     = (CairoColor*)&colors->shade[1];
	border = (CairoColor*)&colors->shade[6];
	ul_shade (bg, &bg_shade, 0.8);

	// Draw shadow 
	aqualooks_rounded_rectangle2 (cr, height-1, 0 + height/2, width-height*2+2, height);
	pattern = cairo_pattern_create_linear (height, 0, height, height);
	cairo_pattern_add_color_stop_rgba (pattern, 0,   bg_shade.r, bg_shade.g, bg_shade.b, 0.7);
	cairo_pattern_add_color_stop_rgba (pattern, 1.0, bg->r,      bg->g,      bg->b,      0.7);

	cairo_set_source (cr, pattern);
	cairo_fill (cr);
	cairo_pattern_destroy (pattern);
	
	/* Draw inner border */
	cairo_set_line_width (cr, 1.5);
	aqualooks_rounded_rectangle2 (cr, height-1, 0 + height/2, width-height*2+2, height);
	cairo_set_source_rgb (cr, border->r, border->g, border->b);
	cairo_stroke (cr);

	cairo_set_source_rgba (cr, border->r, border->g, border->b, 0.3);
	cairo_set_line_width (cr, 2.5);
	cairo_move_to   (cr, height+3, 2.5);
	cairo_line_to   (cr, width-height-3, 2.5);
	cairo_stroke (cr);


	/* Draw border */
	cairo_set_line_width (cr, 2);
	cairo_rectangle (cr, 0, 0, width, height);
	cairo_set_source_rgb (cr, border->r, border->g, border->b);
	cairo_stroke (cr);
}

void
aqualooks_draw_scrollbar_stepper (cairo_t *cr,
                                   const AqualooksColors           *colors,
                                   const WidgetParameters           *widget,
                                   const ScrollBarParameters        *scrollbar,
                                   const ScrollBarStepperParameters *stepper,
                                   int x, int y, int width, int height)
{
	AqualooksCorners corners = CL_CORNER_NONE;
	const CairoColor *border = &colors->shade[ widget->disabled ? 4 : 6];
	const CairoColor *bg     = &colors->bg[widget->state_type];
	CairoColor  fill = (CairoColor)colors->shade[0];
	CairoColor	shade1, shade2, shade3, shade4;
	cairo_pattern_t *pattern;
	ShadowParameters shadow;
	CairoColor        bg_shade;
	CairoColor        center;


	if (scrollbar->horizontal)
	{
		if (stepper->stepper == CL_STEPPER_A)
			corners = CL_CORNER_TOPLEFT | CL_CORNER_BOTTOMLEFT;
		else if (stepper->stepper == CL_STEPPER_D)
			corners = CL_CORNER_TOPRIGHT | CL_CORNER_BOTTOMRIGHT;
	}
	else
	{
		if (stepper->stepper == CL_STEPPER_A)
			corners = CL_CORNER_TOPLEFT | CL_CORNER_TOPRIGHT;
		else if (stepper->stepper == CL_STEPPER_D)
			corners = CL_CORNER_BOTTOMLEFT | CL_CORNER_BOTTOMRIGHT;
	}
	

	if (scrollbar->horizontal)
	{
		cairo_translate (cr, x, y);
	}
	else
	{
		int tmp = height;
		rotate_mirror_translate (cr, M_PI/2, x, y, FALSE, FALSE);
		height = width;
		width = tmp;
	}


	//cairo_translate (cr, x, y);

	cairo_set_line_width (cr, 1);
	                                     
	if (widget->prelight) {
		ul_shade (&fill, &fill, 1.1);
	}
		
	cairo_set_line_width (cr, 1);
	
	ul_shade (&fill, &shade1, 1.1);
	ul_shade (&fill, &shade2, 1.05);
	ul_shade (&fill, &shade3, 0.85);
	ul_shade (&fill, &shade4, 1.0);


	pattern = cairo_pattern_create_linear (1, 1, 1, height-4);
	cairo_pattern_add_color_stop_rgb (pattern, 0,   shade1.r, shade1.g, shade1.b);
	cairo_pattern_add_color_stop_rgb (pattern, 0.5, shade2.r, shade2.g, shade2.b);
	cairo_pattern_add_color_stop_rgb (pattern, 1,   shade4.r, shade4.g, shade4.b);	
	cairo_pattern_add_color_stop_rgb (pattern, 0.5, shade3.r,  shade3.g,  shade3.b);
	cairo_rectangle (cr, 0, 0, width, height);

	cairo_set_source (cr, pattern);
	cairo_fill (cr);
	cairo_pattern_destroy (pattern);
	
	cairo_rectangle (cr, 1.5, 1.5, width-3, height-3);
	cairo_set_source_rgba (cr, 1, 1, 1, 0.6);
	cairo_stroke (cr);


}

void
aqualooks_draw_scrollbar_slider (cairo_t *cr,
                                   const AqualooksColors          *colors,
                                   const WidgetParameters          *widget,
                                   const ScrollBarParameters       *scrollbar,
                                   int x, int y, int width, int height)
{

	double radius;

	radius = 7.00;


	if (scrollbar->junction & CL_JUNCTION_BEGIN)
	{
		if (scrollbar->horizontal)
		{
			x -= 1;
			width += 1;
			radius = 4.40;
		}
		else
		{
			y -= 1;
			height += 1;
			radius = 7.00;
		}
	}
	if (scrollbar->junction & CL_JUNCTION_END)
	{
		if (scrollbar->horizontal)
			width += 1;
		else
			height += 1;
	}
	
	CairoColor  fill;
	CairoColor  fill2;
	
	if (scrollbar->has_color)
		fill    = scrollbar->color;
	else 
		fill    = colors->bg[0];
		fill2   = colors->bg[CL_STATE_PRELIGHT];
		
	CairoColor *border  = (CairoColor*)&colors->shade[6];
	
	CairoColor  hilight;

	if (scrollbar->horizontal)
		cairo_translate (cr, x, y);	

	else
	{
		int tmp = height;
		rotate_mirror_translate (cr, M_PI/2, x, y, FALSE, FALSE);
		height = width;
		width = tmp;
	}

	if (widget->prelight)
		ul_shade (&fill2, &fill, 1.05);

	cairo_set_line_width (cr, 1);

	ul_shade (&fill, &hilight, 1.1);
	
	aqualooks_rounded_rectangle2 (cr, 0.5, 0.5 + (height-1)/2, width-1, height-1);
	cairo_set_source_rgb (cr, border->r, border->g, border->b);		
	cairo_stroke(cr); 

	cairo_set_source_rgb (cr, fill.r, fill.g, fill.b);
	aqualooks_rounded_rectangle2 (cr, 1.25, 1.25 + (height-2.5)/2, width-2.5, height-2.5);
	cairo_fill(cr);


	ShadowParameters shadow = { CL_CORNER_ALL, CL_SHADOW_IN };
	aqualooks_draw_push_button (cr, colors, widget, &shadow, 0.5,0.5,width-1,height-1, 7);


	//the white inner border
	aqualooks_rounded_rectangle2 (cr, 1.8, 1.8 + (height-3.6)/2, width-3.6, height-3.6);
	cairo_set_source_rgb (cr, hilight.r, hilight.g, hilight.b);
	cairo_stroke(cr);
	
	cairo_set_source_rgb (cr, fill.r, fill.g, fill.b);
	cairo_move_to (cr, 1, height-(radius-1));
	cairo_curve_to (cr, 1, height-(radius-1), width*1/2, height, width-2, height-(radius-1));
	cairo_stroke (cr);

	cairo_move_to (cr, 1.5, height-(radius-0.5));
	cairo_curve_to (cr, 1.5, height-(radius-0.5), width*1/2, height-1, width-3, height-(radius-0.5));
	cairo_stroke (cr);
	
}

void
aqualooks_draw_statusbar (cairo_t *cr,
                           const AqualooksColors          *colors,
                           const WidgetParameters          *widget,
                           int x, int y, int width, int height)
{
	cairo_set_line_width  (cr, 0.5);

	cairo_translate       (cr, x, y+0.5);
	cairo_move_to         (cr, 0, 0);
	cairo_line_to         (cr, width, 0);
	cairo_set_source_rgba (cr, 0, 0, 0, 0.2);
	cairo_stroke          (cr);

	cairo_translate       (cr, 0, 1);
	cairo_move_to         (cr, 0, 0);
	cairo_line_to         (cr, width, 0);
	cairo_set_source_rgba (cr, 1, 1, 1, 0.8);
	cairo_stroke          (cr);
}

void
aqualooks_draw_menu_frame (cairo_t *cr,
                            const AqualooksColors          *colors,
                            const WidgetParameters          *widget,
                            int x, int y, int width, int height)
{
	const CairoColor *border = &colors->shade[5];

	// ------------
	CairoColor *bg = (CairoColor*)&colors->bg[CL_STATE_INSENSITIVE];
	cairo_set_source_rgba (cr, bg->r, bg->g, bg->b, 0.15);
	double		tile_pos = 0;

	while (tile_pos <= height)	{
	
		cairo_move_to   (cr, 0, tile_pos);
		cairo_line_to   (cr, width, tile_pos);
		tile_pos += 5;
		cairo_stroke (cr);
	}
	
	//-------------

	cairo_translate      (cr, x, y);
	cairo_set_line_width (cr, 0.5);

	cairo_rectangle      (cr, 0.5, 0.5, width-1, height-1);
	cairo_set_source_rgb (cr, border->r, border->g, border->b);
	cairo_stroke         (cr);

}

void
aqualooks_draw_handle (cairo_t *cr,
                        const AqualooksColors          *colors,
                        const WidgetParameters          *widget,
                        const HandleParameters          *handle,
                        int x, int y, int width, int height)
{
	
	const CairoColor *fill  = &colors->bg[widget->state_type];
	
	int num_bars, bar_spacing;
	
	if (handle->type == CL_HANDLE_TOOLBAR)
	{
		num_bars    = 6;
		bar_spacing = 3;
	}
	else if (handle->type == CL_HANDLE_SPLITTER)
	{
		num_bars    = 16;
		bar_spacing = 3;
	}

	if (widget->prelight)
	{
		cairo_rectangle (cr, x, y, width, height);
		cairo_set_source_rgb (cr, fill->r, fill->g, fill->b);
		cairo_fill (cr);
	}
	
	cairo_translate (cr, x+0.5, y+0.5);
	
	cairo_set_line_width (cr, 1);
	
	if (handle->horizontal)
	{
		aqualooks_draw_gripdots (cr, 0, 0, width, height, num_bars, 2, 0.1);
	}
	else
	{
		aqualooks_draw_gripdots (cr, 0, 0, width, height, 2, num_bars, 0.1);
	}
}

static void
aqualooks_draw_normal_arrow (cairo_t *cr, const CairoColor *color,
                              double x, double y, double width, double height)
{
	const int ARROW_WIDTH  = 10.0;
	const int ARROW_HEIGHT = 5.0;

	cairo_set_line_width (cr, 1);
	
	cairo_move_to   (cr, x-ARROW_WIDTH/2, y-ARROW_HEIGHT/2);
	cairo_line_to   (cr, x, y+ARROW_HEIGHT/2);
	cairo_line_to   (cr, x+ARROW_WIDTH/2, y-ARROW_HEIGHT/2);
	
	cairo_set_source_rgb (cr, color->r, color->g, color->b);
	cairo_fill (cr);	
}

static void
aqualooks_draw_combo_arrow (cairo_t *cr, const CairoColor *fill,
                             double x, double y, double width, double height)
{
	const int ARROW_WIDTH   = 7.0;
	const int ARROW_HEIGHT  = 5.0;
	const int ARROW_SPACING = 8;
	
	cairo_set_line_width (cr, 1);

	y -= ARROW_SPACING/2;
	
	cairo_move_to (cr, x-ARROW_WIDTH/2, y+ ARROW_HEIGHT/2);
	cairo_line_to   (cr, x, y-ARROW_HEIGHT/2);
	cairo_line_to   (cr, x+ARROW_WIDTH/2, y+ARROW_HEIGHT/2);
	cairo_set_source_rgb (cr, fill->r, fill->g, fill->b);	
	cairo_fill (cr);
	
	y += ARROW_SPACING;
	
	cairo_move_to (cr, x-ARROW_WIDTH/2, y + -ARROW_HEIGHT/2);
	cairo_line_to   (cr, x, y+ARROW_HEIGHT/2);
	cairo_line_to   (cr, x+ARROW_WIDTH/2, y-ARROW_HEIGHT/2);
	cairo_set_source_rgb (cr, fill->r, fill->g, fill->b);	
	cairo_fill (cr);
}

static void
_aqualooks_draw_arrow (cairo_t *cr, const CairoColor *color,
                        AqualooksDirection dir, AqualooksArrowType type,
                        double x, double y, double width, double height)
{
	double rotate;
	
	if (dir == CL_DIRECTION_LEFT)
		rotate = M_PI*1.5;
	else if (dir == CL_DIRECTION_RIGHT)
		rotate = M_PI*0.5;
	else if (dir == CL_DIRECTION_UP)
		rotate = M_PI;
	else
		rotate = 0;
	
	if (dir == CL_DIRECTION_UP || dir == CL_DIRECTION_DOWN)
		x++;
	
	if (type == CL_ARROW_NORMAL)
	{
		rotate_mirror_translate (cr, rotate, x, y, FALSE, FALSE);		
		aqualooks_draw_normal_arrow (cr, color, -0.5, -0.5, width, height);
	}
	else if (type == CL_ARROW_COMBO)
	{
		cairo_translate (cr, x, y);
		aqualooks_draw_combo_arrow (cr, color, -0.5, -0.5, width, height);
	}
}

void
aqualooks_draw_arrow (cairo_t *cr,
                       const AqualooksColors          *colors,
                       const WidgetParameters          *widget,
                       const ArrowParameters           *arrow,
                       int x, int y, int width, int height)
{
	CairoColor black = {0,0,0};
	const CairoColor *color = &black;
	gdouble tx, ty;
	
	if (arrow->direction == CL_DIRECTION_DOWN || arrow->direction == CL_DIRECTION_UP)
	{
		tx = x + width/2;
		ty = (y + height/2) + 0.5;
	}
	else
	{
		tx = (x + width/2) + 0.5;
		ty = y + height/2;
	}
	
	if (widget->disabled)
	{
		_aqualooks_draw_arrow (cr, &colors->shade[0],
		                        arrow->direction, arrow->type,
		                        tx+0.5, ty+0.5, width, height);
		
		color = &colors->shade[4];
	}

	cairo_identity_matrix (cr);
	
	_aqualooks_draw_arrow (cr, color, arrow->direction, arrow->type,
	                        tx, ty, width, height);
}

void
aqualooks_draw_resize_grip (cairo_t *cr,
                             const AqualooksColors          *colors,
                             const WidgetParameters          *widget,
                             const ResizeGripParameters      *grip,
                             int x, int y, int width, int height)
{
	int lx, ly;
 
	cairo_set_line_width (cr, 1);
	
	for (ly=0; ly<4; ly++) // vertically, four rows of dots
	{
		for (lx=0; lx<=ly; lx++) // horizontally
		{
			int ny = (3.5-ly) * 3;
			int nx = lx * 3;

			cairo_set_source_rgba (cr, 1, 1, 1, 1.2);
			cairo_rectangle (cr, x+width-nx-1, y+height-ny-1, 2, 2);
			cairo_fill (cr);

			cairo_set_source_rgba (cr, 0, 0, 0, 0.4);
			cairo_rectangle (cr, x+width-nx-1, y+height-ny-1, 1, 1);
			cairo_fill (cr);
		}
	}
}

void
aqualooks_draw_checkbox (cairo_t *cr,
                          const AqualooksColors  *colors,
                          const WidgetParameters  *widget,
                          const CheckboxParameters *checkbox,
                          int x, int y, int width, int height)
{
	CairoColor border, top, bottom;
	cairo_pattern_t *pattern;
	gboolean draw_bullet = (checkbox->shadow_type == GTK_SHADOW_IN);
	
	cairo_set_line_width (cr, 1.0);


	if (!checkbox->in_menu)
	{

		if ( (checkbox->shadow_type == CL_SHADOW_IN || 
		      checkbox->shadow_type == CL_SHADOW_ETCHED_IN || 
		      widget->active) && !widget->disabled && !checkbox->in_cell )
		{
			border = top = bottom = colors->spot[1];
			
			if (widget->prelight)
			{
				ul_shade (&top, &top, 1.1);
				ul_shade (&border, &border, 0.8);
			}
			
			ul_shade (&top, &top, 1.2);
			ul_shade (&bottom, &bottom, 0.9);
			ul_shade (&border, &border, 0.6);
		}
		else if (widget->disabled)
		{
			if (checkbox->in_cell) {
				border = top = bottom = colors->shade[3];
			} else {
				border = top = bottom = colors->shade[3];
			}
		}	
		else
		{
			top = bottom = colors->base[widget->state_type];
			
			if (widget->prelight)
			{
				ul_shade (&top, &top, 1.2);
				border = colors->spot[2];
				
			}
			else
			{
				border = colors->base[widget->state_type];
				ul_shade(&border, &border, 0.4);
			}
			
			ul_shade (&top, &top, 0.98);
			ul_shade (&bottom, &bottom, 0.85);
		}
		
		if (widget->state_type != GTK_STATE_INSENSITIVE)
		{
			// glow
			if ((widget->prelight || (widget->active && !draw_bullet)) && !checkbox->in_cell) 
			{
				const CairoColor *glow = &colors->spot[0];
				aqualooks_rounded_rectangle (cr, x+1.5, y+1.5, width - 1, height - 1, 2, widget->corners);
				cairo_set_source_rgba (cr, glow->r, glow->g, glow->b, 0.5);
				cairo_stroke (cr);
			}
			
			// shadow
			if (checkbox->in_cell) {
				aqualooks_rounded_rectangle (cr, x+0.5, y+0.5, width, height, 1, widget->corners);
			} else {
				aqualooks_rounded_rectangle (cr, x+3.5, y+3.5, width - 3, height - 3, 1, widget->corners);
			}
			cairo_set_source_rgba (cr, 0., 0., 0., 0.2);
			cairo_stroke (cr);
		
			if (checkbox->in_cell) {
				aqualooks_rounded_rectangle (cr, x+0.5, y+0.5, width, height, 1, widget->corners);
			} else {
				aqualooks_rounded_rectangle (cr, x+2.5, y+2.5, width - 3, height - 3, 1, widget->corners);
			}
			pattern = cairo_pattern_create_linear (x, y, x+width, y+height);
			cairo_pattern_add_color_stop_rgb (pattern, 0.0, 1.0, 1.0, 1.0);
			cairo_pattern_add_color_stop_rgb (pattern, 0.4, top.r, top.g, top.b);
			cairo_pattern_add_color_stop_rgb (pattern, 1.0, bottom.r, bottom.g, bottom.b);
			cairo_set_source (cr, pattern);
			cairo_fill (cr);
			cairo_pattern_destroy (pattern);
		}
		
		if (checkbox->in_cell) {
			aqualooks_rounded_rectangle (cr, x+0.5, y+0.5, width, height, 1, widget->corners);
		} else {
			aqualooks_rounded_rectangle (cr, x+2.5, y+2.5, width - 3, height - 3, 1, widget->corners);
		}
		
		cairo_set_source_rgb (cr, border.r, border.g, border.b);
		cairo_stroke (cr);
		
		if (checkbox->in_cell) {
			aqualooks_rounded_rectangle (cr, x+1.5, y+1.5, width - 2, height - 2, 1, widget->corners);
		} else {
			aqualooks_rounded_rectangle (cr, x+3.5, y+3.5, width - 5, height - 5, 1, widget->corners);
		}
		cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.4);
		cairo_stroke (cr);
		
	}
	
	/* To offset this checkmark, one should probably use a translation */
	if (draw_bullet)
	{
		if (checkbox->in_cell)
		{
			cairo_translate (cr, -2, -2);
		}
		else if (checkbox->in_menu)
		{
			cairo_translate (cr, -2, -2);
		}
	
		cairo_move_to (cr, x + 4, y + 8);
		cairo_rel_line_to (cr,   5,   4);
		cairo_rel_curve_to (cr,  1.4,  -5,   -1,  -1,   5.7,  -12.5);
		cairo_rel_curve_to (cr, -4,   4,  -4,   4,  -6.7,    9.3);
		cairo_rel_line_to (cr,  -2.3,  -2.5);
		
		
		cairo_set_source_rgb (cr, colors->text[widget->state_type].r,
		                          colors->text[widget->state_type].g,
		                          colors->text[widget->state_type].b);
		cairo_fill (cr);
	}
	else if (checkbox->shadow_type == GTK_SHADOW_ETCHED_IN)
	{
		int c_x = x + (width/2) - 2;
		int c_y = y + (height/2);

		cairo_rectangle (cr, c_x, c_y, 6, 2);
		cairo_set_source_rgb (cr, colors->text[widget->state_type].r,
		                          colors->text[widget->state_type].g,
		                          colors->text[widget->state_type].b);

		cairo_fill(cr);

	}
}

void
aqualooks_draw_radiobutton (cairo_t *cr,
                             const AqualooksColors  *colors,
                             const WidgetParameters  *widget,
                             const CheckboxParameters *checkbox,
                             int x, int y, int width, int height)
{
	CairoColor border, top, bottom;
	cairo_pattern_t *pattern;
	gboolean draw_bullet = (checkbox->shadow_type == CL_SHADOW_IN || checkbox->shadow_type == CL_SHADOW_ETCHED_IN);
	
	cairo_set_line_width (cr, 1.0);
	
	if (checkbox->in_menu)
	{
		cairo_translate (cr, -2, -1);
	}

	if ( (checkbox->shadow_type == CL_SHADOW_IN || 
	      checkbox->shadow_type == CL_SHADOW_ETCHED_IN || 
	      widget->active) && !widget->disabled )
	{
		border = top = bottom = colors->spot[1];
		
		if (widget->prelight)
		{
			ul_shade (&top, &top, 1.1);
			ul_shade (&border, &border, 0.8);
		}
		
		ul_shade (&top, &top, 1.2);
		ul_shade (&bottom, &bottom, 0.9);
		ul_shade (&border, &border, 0.6);
	}
	else if (widget->disabled)
	{
		border = top = bottom = colors->shade[3];
	}
	else
	{		
		top = bottom = colors->base[widget->state_type];
		
		if (widget->prelight)
		{
			ul_shade (&top, &top, 1.2);
			border = colors->spot[2];
			
		}
		else
		{
			border = colors->base[widget->state_type];
			ul_shade(&border, &border, 0.4);
		}
		
		ul_shade (&top, &top, 0.98);
		ul_shade (&bottom, &bottom, 0.85);
	}
	
	if ( widget->state_type != GTK_STATE_INSENSITIVE )
	{
		// glow
		if (widget->prelight || (widget->active && !draw_bullet)) 
		{
			const CairoColor *glow = &colors->spot[0];
			cairo_arc (cr, x+width/2.0 + 0.5, y+height/2.0 + 0.5, width/2.0, 0, 2 * M_PI);
			cairo_set_source_rgba (cr, glow->r, glow->g, glow->b, 1.0);
			cairo_stroke (cr);
		}
	
		// shadow
		cairo_arc (cr, x+width/2.0+1, y+height/2.0+1, width/2.0 - 1.0, 0, 2 * M_PI);
		cairo_set_source_rgba (cr, 0., 0., 0., 0.2);
		cairo_stroke (cr);
		
		cairo_arc (cr, x+width/2., y+height/2.0, width/2.0 - 1.0, 0, 2 * M_PI);	
		pattern = cairo_pattern_create_linear (x, y, x+width, y+height);
		cairo_pattern_add_color_stop_rgb (pattern, 0.0, 1.0, 1.0, 1.0);
		cairo_pattern_add_color_stop_rgb (pattern, 0.4, top.r, top.g, top.b);
		cairo_pattern_add_color_stop_rgb (pattern, 1.0, bottom.r, bottom.g, bottom.b);
		cairo_set_source (cr, pattern);
		cairo_fill(cr);
		cairo_pattern_destroy (pattern);
	}
	cairo_translate (cr, 0.5, 0.5);
	cairo_arc (cr, x+width/2.0, y+height/2.0, width/2.0 - 1.0, 0, 2 * M_PI);
	
	cairo_set_source_rgb (cr, border.r, border.g, border.b);
	cairo_stroke (cr);
	
	cairo_arc (cr, x+width/2., y+height/2.0, width/2.0 - 2.0, 0, 2 * M_PI);
	cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.4);
	cairo_stroke (cr);
	
	// draw the bullet
	if (draw_bullet)
	{
		cairo_arc (cr, x+width/2., y+height/2., 2.5, 0, 2 * M_PI);
		if (widget->disabled)
		{
			cairo_set_source_rgb (cr, colors->text[widget->state_type].r,
			                          colors->text[widget->state_type].g,
			                          colors->text[widget->state_type].b);
		}
		else
		{
			pattern = cairo_pattern_create_radial (x+width/2. - 1.25, y+height/2. - 1.25, 0.1, x+width/2. - 1.25, y+height/2. - 1.25, 4);
			cairo_pattern_add_color_stop_rgb (pattern, 0.0, 0.4, 0.4, 0.4);
			cairo_pattern_add_color_stop_rgb (pattern, 1.0, colors->text[widget->state_type].r,
			                                                colors->text[widget->state_type].g,
			                                                colors->text[widget->state_type].b);
			cairo_set_source (cr, pattern);
			cairo_pattern_destroy (pattern);
		}
		cairo_fill_preserve (cr);
	}
}

void
aqualooks_draw_prelightbox (cairo_t *cr,
                              const AqualooksColors         *colors,
                              const WidgetParameters          *widget,
                              int x, int y, int width, int height)
{
	const CairoColor  *bg = &colors->bg[CL_STATE_PRELIGHT];

	aqualooks_rounded_rectangle (cr, x, y, width, height, 5.0, widget->corners);
	cairo_set_source_rgb (cr, bg->r, bg->g, bg->b);
	cairo_fill (cr);
}

void
aqualooks_draw_tooltip (cairo_t *cr,
                          const AqualooksColors         *colors,
                          const WidgetParameters          *widget,
                          int x, int y, int width, int height)
{
	const CairoColor  *bg = &colors->bg[CL_STATE_NORMAL];
	CairoColor        tmp;
	cairo_pattern_t  *pattern;
		
	cairo_set_line_width (cr, 1.0);
		
	ul_shade (bg, &tmp, 0.9);
	
	cairo_translate (cr, x, y);
	
	cairo_rectangle (cr, 0.5, 0.5, width-1, height-1);
	
	pattern = cairo_pattern_create_linear (0, 0, width, 0);
	cairo_pattern_add_color_stop_rgb (pattern, 0.0, tmp.r, tmp.g, tmp.b);
	cairo_pattern_add_color_stop_rgb (pattern, 0.5, bg->r, bg->g, bg->b);
	cairo_pattern_add_color_stop_rgb (pattern, 1.0, tmp.r, tmp.g, tmp.b);
	cairo_set_source (cr, pattern);
	cairo_fill_preserve (cr);
	cairo_pattern_destroy (pattern);
	
	cairo_set_source_rgb (cr, 0, 0, 0);
	cairo_stroke (cr);
}

void
aqualooks_draw_gdm_window (cairo_t *cr,
                             const AqualooksColors         *colors,
                             const WidgetParameters          *widget,
                             int x, int y, int width, int height)
{
	const CairoColor  *bg = &colors->bg[CL_STATE_NORMAL];
	CairoColor        tmp;
	cairo_pattern_t  *pattern;

	cairo_translate (cr, x, y);

	ul_shade (bg, &tmp, 0.9);

	cairo_rectangle (cr, 0, 0, width, height);
	pattern = cairo_pattern_create_linear (0, 0, width, 0);
	cairo_pattern_add_color_stop_rgb (pattern, 0.0, tmp.r, tmp.g, tmp.b);
	cairo_pattern_add_color_stop_rgb (pattern, 0.5, bg->r, bg->g, bg->b);
	cairo_pattern_add_color_stop_rgb (pattern, 1.0, tmp.r, tmp.g, tmp.b);
	cairo_set_source (cr, pattern);
	cairo_fill (cr);
	cairo_pattern_destroy (pattern);

	// ------------
	CairoColor *bg2 = (CairoColor*)&colors->bg[CL_STATE_INSENSITIVE];
	cairo_set_source_rgba (cr, bg2->r, bg2->g, bg2->b, 0.15);
	double		tile_pos = 0;

	while (tile_pos <= height)	{
	
		cairo_move_to   (cr, 0, tile_pos);
		cairo_line_to   (cr, width, tile_pos);
		tile_pos += 5;
		cairo_stroke (cr);
	}
	
	//-------------

}

void
aqualooks_draw_gdm_menu_frame (cairo_t *cr,
                                 const AqualooksColors         *colors,
                                 const WidgetParameters          *widget,
                                 int x, int y, int width, int height)
{
	cairo_pattern_t  *pattern;
	CairoColor        tmp1, tmp2;
	const CairoColor *spot = &colors->spot[1];

	cairo_translate      (cr, x, y);
	cairo_set_line_width (cr, 1);
	cairo_rectangle      (cr, 0.5, 0.5, width-1, height-1);
	cairo_set_source_rgb (cr, 0, 0, 0);
	cairo_stroke_preserve (cr);

	ul_shade (spot, &tmp1, 1.6);
	ul_shade (spot, &tmp2, 1.5);

	pattern = cairo_pattern_create_linear (0, 0, width, 0);
	cairo_pattern_add_color_stop_rgb (pattern, 0.0, tmp1.r, tmp1.g, tmp1.b);
	cairo_pattern_add_color_stop_rgb (pattern, 1.0, tmp2.r, tmp2.g, tmp2.b);
	cairo_set_source (cr, pattern);
	cairo_fill (cr);
	cairo_pattern_destroy (pattern);
}

void
aqualooks_draw_list_selection (cairo_t *cr,
                                 const AqualooksColors         *colors,
                                 const WidgetParameters          *widget,
                                 int x, int y, int width, int height)
{
	cairo_pattern_t *pattern;
	CairoColor       lower_color;
	CairoColor       upper_color;

	cairo_translate (cr, x, y);

	if (widget->focus)
		upper_color = colors->base[widget->state_type];
	else
		upper_color = colors->base[GTK_STATE_ACTIVE];

	ul_shade (&upper_color, &lower_color, 0.9);

	pattern = cairo_pattern_create_linear (0, 0, 0, height);
	cairo_pattern_add_color_stop_rgb (pattern, 0.0, upper_color.r,
	                                                upper_color.g,
	                                                upper_color.b);
	cairo_pattern_add_color_stop_rgb (pattern, 1.0, lower_color.r,
	                                                lower_color.g,
	                                                lower_color.b);

	cairo_set_source (cr, pattern);
	cairo_rectangle  (cr, 0, 0, width, height);
	cairo_fill       (cr);

	cairo_pattern_destroy (pattern);
}

