#ifndef AQUALOOKS_DRAW_H
#define AQUALOOKS_DRAW_H

#include "aqualooks_types.h"

#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include <cairo.h>

void aqualooks_rounded_rectangle     (cairo_t *cr,
                                       double x, double y, double w, double h,
                                       double radius, uint8 corners);

void dyndyn_draw_gradients_inwards   (cairo_t *cr,
					int xoffset,
					int yoffset,
					int width,
					int height, 
					double radius, 
					uint8 corners);

void aqualooks_draw_button           (cairo_t *cr,
                                       const AqualooksColors      *colors,
                                       const WidgetParameters      *widget,
                                       const ShadowParameters      *shadow,
                                       int x, int y, int width, int height,
				       double radius);

void
aqualooks_draw_push_button (cairo_t *cr,
                        const AqualooksColors *colors,
                        const WidgetParameters *params,
                        const ShadowParameters *shadow,
                        int x, int y, int width, int height, 
			double radius);

void aqualooks_draw_scale_trough     (cairo_t *cr,
                                       const AqualooksColors      *colors,
                                       const WidgetParameters      *widget,
                                       const SliderParameters      *slider,
                                       int x, int y, int width, int height);

void aqualooks_draw_progressbar_trough (cairo_t *cr,
                                       const AqualooksColors      *colors,
                                       const WidgetParameters      *widget,
                                       const ProgressBarParameters *progressbar,
                                       int x, int y, int width, int height);


void aqualooks_draw_progressbar_fill (cairo_t *cr,
                                       const AqualooksColors      *colors,
                                       const WidgetParameters      *widget,
                                       const ProgressBarParameters *progressbar,
                                       int x, int y, int width, int height, gint offset);

void aqualooks_draw_slider_button    (cairo_t *cr,
                                       const AqualooksColors     *colors,
                                       const WidgetParameters     *widget,
                                       const SliderParameters     *slider,
                                       int x, int y, int width, int height);

void aqualooks_draw_entry            (cairo_t *cr,
                                       const AqualooksColors     *colors,
                                       const WidgetParameters     *widget,
                                       int x, int y, int width, int height);

void aqualooks_draw_spinbutton       (cairo_t *cr,
                                       const AqualooksColors     *colors,
                                       const WidgetParameters     *widget,
                                       int x, int y, int width, int height);

void aqualooks_draw_spinbutton_down  (cairo_t *cr,
                                       const AqualooksColors     *colors,
                                       const WidgetParameters     *widget,
                                       int x, int y, int width, int height);

void aqualooks_draw_optionmenu       (cairo_t *cr,
                                       const AqualooksColors     *colors,
                                       const WidgetParameters     *widget,
                                       const OptionMenuParameters *optionmenu,
                                       int x, int y, int width, int height);
								   
void aqualooks_draw_inset            (cairo_t *cr,
                                       int width, int height,
                                       double radius, uint8 corners);

void aqualooks_draw_menubar          (cairo_t *cr,
                                       const AqualooksColors     *colors,
                                       const WidgetParameters     *widget,
				       boolean fixed,
                                       int x, int y, int width, int height);

void aqualooks_draw_tab              (cairo_t *cr,
                                       const AqualooksColors     *colors,
                                       const WidgetParameters     *widget,
                                       const TabParameters        *tab,
                                       int x, int y, int width, int height);

void aqualooks_draw_frame            (cairo_t *cr,
                                       const AqualooksColors     *colors,
                                       const WidgetParameters     *widget,
                                       const FrameParameters      *frame,
                                       int x, int y, int width, int height);

void aqualooks_draw_separator        (cairo_t *cr,
                                       const AqualooksColors     *colors,
                                       const WidgetParameters     *widget,
                                       const SeparatorParameters  *separator,
                                       int x, int y, int width, int height);

void aqualooks_draw_list_view_header (cairo_t *cr,
                                       const AqualooksColors         *colors,
                                       const WidgetParameters         *widget,
                                       const ListViewHeaderParameters *header,
                                       int x, int y, int width, int height);

void aqualooks_draw_toolbar          (cairo_t *cr,
                                       const AqualooksColors         *colors,
                                       const WidgetParameters         *widget,
                                       int x, int y, int width, int height, 
					boolean gradient);

void aqualooks_draw_menuitem         (cairo_t *cr,
                                       const AqualooksColors         *colors,
                                       const WidgetParameters         *widget,
                                       int x, int y, int width, int height);

void aqualooks_draw_scrollbar_stepper (cairo_t *cr,
                                        const AqualooksColors           *colors,
                                        const WidgetParameters           *widget,
                                        const ScrollBarParameters        *scrollbar,
                                        const ScrollBarStepperParameters *stepper,
                                        int x, int y, int width, int height);

void aqualooks_draw_scrollbar_slider (cairo_t *cr,
                                       const AqualooksColors          *colors,
                                       const WidgetParameters          *widget,
                                       const ScrollBarParameters       *scrollbar,
                                       int x, int y, int width, int height);

void aqualooks_draw_scrollbar_trough (cairo_t *cr,
                                       const AqualooksColors          *colors,
                                       const WidgetParameters          *widget,
                                       const ScrollBarParameters       *scrollbar,
                                       int x, int y, int width, int height);

void aqualooks_draw_statusbar        (cairo_t *cr,
                                       const AqualooksColors          *colors,
                                       const WidgetParameters          *widget,
                                       int x, int y, int width, int height);

void aqualooks_draw_menu_frame       (cairo_t *cr,
                                       const AqualooksColors          *colors,
                                       const WidgetParameters          *widget,
                                       int x, int y, int width, int height);

void aqualooks_draw_handle           (cairo_t *cr,
                                       const AqualooksColors          *colors,
                                       const WidgetParameters          *widget,
                                       const HandleParameters          *handle,
                                       int x, int y, int width, int height);

void aqualooks_draw_arrow            (cairo_t *cr,
                                       const AqualooksColors          *colors,
                                       const WidgetParameters          *widget,
                                       const ArrowParameters           *arrow,
                                       int x, int y, int width, int height);

void aqualooks_draw_resize_grip      (cairo_t *cr,
                                       const AqualooksColors          *colors,
                                       const WidgetParameters          *widget,
                                       const ResizeGripParameters      *grip,
                                       int x, int y, int width, int height);

void aqualooks_draw_checkbox         (cairo_t *cr,
                                       const AqualooksColors          *colors,
                                       const WidgetParameters          *widget,
                                       const CheckboxParameters        *checkbox,
                                       int x, int y, int width, int height);

void aqualooks_draw_radiobutton      (cairo_t *cr,
                                       const AqualooksColors         *colors,
                                       const WidgetParameters          *widget,
                                       const CheckboxParameters        *checkbox,
                                       int x, int y, int width, int height);

void aqualooks_draw_prelightbox      (cairo_t *cr,
                                        const AqualooksColors         *colors,
                                        const WidgetParameters          *widget,
                                        int x, int y, int width, int height);

void aqualooks_draw_tooltip          (cairo_t *cr,
                                        const AqualooksColors         *colors,
                                        const WidgetParameters          *widget,
                                        int x, int y, int width, int height);

void aqualooks_draw_gdm_window       (cairo_t *cr,
                                        const AqualooksColors         *colors,
                                        const WidgetParameters          *widget,
                                        int x, int y, int width, int height);

void aqualooks_draw_gdm_menu_frame   (cairo_t *cr,
                                        const AqualooksColors         *colors,
                                        const WidgetParameters          *widget,
                                        int x, int y, int width, int height);

void aqualooks_draw_list_selection   (cairo_t *cr,
                                        const AqualooksColors         *colors,
                                        const WidgetParameters          *widget,
                                        int x, int y, int width, int height);

#endif /* AQUALOOKS_DRAW_H */
